#import json, os, pickle, Levenshtein
import json, os, pickle, distance



basepath = os.path.dirname(__file__)
listspath = os.path.join(basepath,"..","lists\\")

def distance_functionLV(str1,str2):    # Levenshtein distance
    return distance.levenshtein(str1,str2)

def distance_functionHM(str1,str2):    # Hamming distance
    return distance.hamming(str1,str2, normalized=True)

def distance_functionJC(str1,str2):    # Jaccard metrices
    return distance.jaccard(str1,str2)

def load_words(filename):#nome do arquivo, sem extensão
    try:
        filepath = (os.path.join(basepath,"..","dicts\\", filename+".json"))
        with open(filepath,"r") as dictionary:
            words = json.load(dictionary)
            return words
    except Exception as e:
        return str(e)

def pickle_dump(tree, filename):
    try:
        filepath = (os.path.join(basepath,"..","dicts\\", filename+".txt"))
        with open(filepath, "wb") as pickling:
            pickle.dump(tree, pickling)
    except Exception as e:
        return str(e)


def pickle_load(filename):
    filepath = (os.path.join(basepath, "..", "dicts\\", filename + ".txt"))
    with open(filepath, "rb") as pickling:
        return pickle.load(pickling)


from core.functions import *

#LOADS THE JSON DICTIONARY, CREATES A TREE, ADDS THE DICTIONARY TO THE TREE, AND DUMPS IT INTO A PICKLE OBJECT
#pickle_dump(BKTree(load_words("english_dict")),"english_dict_pickle")

def run():
    tree = pickle_load("english_dict_pickle")
    while True:
        user_input = input("Write the word you want spellchecked:")
        if user_input != "":
            print(tree.find(user_input, 3))
        else:
            break

if __name__ == "__main__":
    run()


from core.functions import *
import cProfile, os, pickle, Levenshtein, distance


def test():
    for i in range(0,1000000):
        print(Levenshtein.distance("everything","doesnotmatter"))

def test2():
    for i in range(0,1000000):
        print(distance.levenshtein("everything","doesnotmatter"))

def test3():
    tree = pickle_load("english_dict_pickle")
    print(tree.find("zero", 1))

cProfile.run('test3()')




#TESTING WITHOUT TREE
#import distance
#test = load_words("english_dict")
#for word in test:
#    if distance.levenshtein(word,"zero") <= 1:
#        print(word)


from collections import deque
from core.functions import distance_functionLV
from core.functions import distance_functionHM
from core.functions import distance_functionJC


class BKTree(object):
    def __init__(self, items_list=[], items_dict={}):
        self.tree = None

        _add = self.add
        for item in items_list:
            _add(item)

        for item in items_dict:
            _add(item)

    def add(self, item):
        node = self.tree
        if node is None:
            self.tree = (item, {})
            return

        while True:
            parent, children = node
            distance = distance_functionLV(item, parent)
            node = children.get(distance)
            if node is None:
                children[distance] = (item, {})
                break

    def findLV(self, item, n):
        if self.tree is None:
            return []

        candidates = deque([self.tree])
        found = []

        while candidates:
            candidate, children = candidates.popleft()
            candidate = candidate.lower()                                               # Change all letters to lower case
            distanceLV = distance_functionLV(candidate, item)                               # Ganti here so LV, HM and JC included
            #distance = distance_functionHM(candidate, item)                               # Ganti here so LV, HM and JC included
            #distance = distance_functionJC(candidate, item)                               # Ganti here so LV, HM and JC included

            if distanceLV <= n:
                found.append((distanceLV, candidate))

            if children:
                lower = distanceLV - n
                upper = distanceLV + n
                candidates.extend(c for d, c in children.items() if lower <= d <= upper)

        found.sort()
        return found
    
    def findHM(self, item, n):
        if self.tree is None:
            return []

        candidates = deque([self.tree])
        found = []

        while candidates:
            candidate, children = candidates.popleft()
            candidate = candidate.lower()                                               # Change all letters to lower case
            #distanceLV = distance_functionLV(candidate, item)                               # Ganti here so LV, HM and JC included
            distanceHM = distance_functionHM(candidate, item)                               # Ganti here so LV, HM and JC included
            #distanceJC = distance_functionJC(candidate, item)                               # Ganti here so LV, HM and JC included

            if distanceHM <= n:
                found.append((distanceHM, candidate))

            if children:
                lower = distanceHM - n
                upper = distanceHM + n
                candidates.extend(c for d, c in children.items() if lower <= d <= upper)

        found.sort()
        return found


    def __iter__(self):
        if self.tree is None:
            return

        candidates = deque([self.tree])

        while candidates:
            candidate, children = candidates.popleft()
            yield candidate
            candidates.extend(children.values())